# combo - content management system
# Copyright (C) 2014-2017  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.http import Http404
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import RedirectView

from combo.data.models import CellBase, ConfigJsonCell
from combo.data.library import get_cell_class
from combo.public.views import render_cell

from .models import DashboardCell, Tile


class DashboardAddTileView(RedirectView):
    permanent = False

    def get_redirect_url(self, cell_reference):
        dashboard = DashboardCell.objects.all()[0]

        cell = CellBase.get_cell(cell_reference)
        if not cell.page.is_visible(self.request.user):
            raise PermissionDenied()
        if not cell.is_visible(self.request.user):
            raise PermissionDenied()
        cell.pk = None
        cell.page = dashboard.page
        cell.placeholder = '_dashboard'
        cell.save()

        tile = Tile(dashboard=dashboard,
                cell=cell,
                user=self.request.user)
        tile.order = 0
        tile.save()

        return dashboard.page.get_online_url()

dashboard_add_tile = DashboardAddTileView.as_view()


class DashboardRemoveTileView(RedirectView):
    permanent = False

    def get_redirect_url(self, cell_reference):
        cell = CellBase.get_cell(cell_reference)
        try:
            tile = Tile.get_by_cell(cell)
        except Tile.DoesNotExist:
            raise Http404()
        if tile.user != self.request.user:
            raise PermissionDenied()
        dashboard = tile.dashboard
        tile.delete()
        cell.delete()
        return dashboard.page.get_online_url()

dashboard_remove_tile = DashboardRemoveTileView.as_view()


@csrf_exempt
def dashboard_auto_tile(request, *args, **kwargs):
    dashboard = DashboardCell.objects.all()[0]
    cell = ConfigJsonCell(key=kwargs.get('key'), order=1,
            page_id=dashboard.page_id, placeholder='_auto_tile')
    cell.parameters = json.loads(request.body)
    if request.user.is_authenticated():
        # save cell if the user is connected, so it can be added to the
        # dashboard
        cell.save()
    return render_cell(request, cell=cell)
